@@ -16,7 +16,7 @@ Readonly my $EMPTY_ARRAY => -1;
 # Glib::ParamSpec->object in Gscan2pdf::Dialog::Scan
 use Glib::Object::Subclass Glib::Object::;
 
-our $VERSION = '1.8.8';
+our $VERSION = '1.8.9';
 
 my $units = qr{(pel|bit|mm|dpi|%|us)}xsm;
 my $EMPTY = q{};
@@ -194,6 +194,56 @@ sub supports_paper {
     return 0;
 }
 
+# returns TRUE if the current options support duplex, even if not currently
+# selected. Alternatively expressed, return FALSE if the scanner is not capable
+# of duplex scanner, or if the capability is inactive.
+
+sub can_duplex {
+    my ($self) = @_;
+    for my $option ( @{ $self->{array} } ) {
+        if (
+            not( defined $option->{cap}
+                and ( $option->{cap} & SANE_CAP_INACTIVE ) )
+          )
+        {
+            if ( defined $option->{name} and $option->{name} =~ /duplex/xsmi ) {
+                return TRUE;
+            }
+            elsif ( defined $option->{constraint_type}
+                and $option->{constraint_type} == SANE_CONSTRAINT_STRING_LIST )
+            {
+                for my $item ( @{ $option->{constraint} } ) {
+                    if ( $item =~ /duplex/xsmi ) {
+                        return TRUE;
+                    }
+                }
+            }
+        }
+    }
+    return FALSE;
+}
+
+# returns TRUE/FALSE if the value is within the tolerance of the given option or
+# not, and undef for options with no value or for an invalid value
+
+sub within_tolerance {
+    my ( $option, $value ) = @_;
+    if ( $option->{constraint_type} == SANE_CONSTRAINT_RANGE ) {
+        if ( defined $option->{constraint}{quant} ) {
+            return (
+                abs( $value - $option->{val} ) <=
+                  $option->{constraint}{quant} / 2 );
+        }
+    }
+    elsif ( $option->{constraint_type} == SANE_CONSTRAINT_STRING_LIST ) {
+        return ( $value eq $option->{val} );
+    }
+    elsif ( $option->{constraint_type} == SANE_CONSTRAINT_WORD_LIST ) {
+        return ( $value == $option->{val} );
+    }
+    return;
+}
+
 # parse the scanimage/scanadf output into an array and a hash
 
 sub _parse_scanimage_output {
